/*
  SV3 - Main class for sv3 type interface ByVac
  Copyright (c) 2013 Jim Spence.  All right reserved.
  www.byvac.com - see terms and conditions for using hardware
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
  Release Notes:

*/  

#include "Arduino.h"
#include "sv3.h"

// =============================================================================
// SV3
// =============================================================================
// =============================================================================
// *****************************************************************************
// constructor is just a connection to NewSoftSerial
// *****************************************************************************
SV3::SV3(uint8_t rxPin, uint8_t txPin, long rbaud, uint8_t raddr, uint8_t rack) : BSerial(rxPin,txPin)
{
    _addr = raddr;
    _ack = rack;
    baud(rbaud);
}

// *****************************************************************************
// set up baud rate, attach serial and initialise any devices
// that may be on the bus
// *****************************************************************************
void SV3::sv3_init()
{
    puts("\r"); delay(100);
    puts("\r"); delay(100);
    puts("\r"); delay(100);    
}

// *****************************************************************************
// check if a singe device is on the bus, return 1 if it is
// *****************************************************************************
int SV3::active()
{
    flush(); // buffer
    puts("\r"); delay(100);
    delay(20);
    if(getch() == '*') return 1;
    else return 0;
}

// *****************************************************************************
// Commands always start with an address, this will send the
// address followed by the string. NOTE it is up to the caller to supply 
// the \r and Wack
// *****************************************************************************
void SV3::send(char *str)
{
    flush();
    putch(_addr);
    puts(str);
}

// *****************************************************************************
// Gets a character from the input buffer with a timeout, if there is no char
// it will return -1
// as set if no ack then delays about 300mS
// *****************************************************************************
char SV3::getch_t()
{
unsigned int j=1000; // timeout
char c; 

    c = 0; // default
    while(j-- > 0) {
        c = getch();
        if(c != -1) break;
        delay(1);
    }
    return c;
}            

// *****************************************************************************
// for commands that just return ACK,
// Does not return anything but will wait for the ACK char or timeout
// *****************************************************************************
void SV3::wack()
{
char k=0;
    while(k != _ack) {
        k = getch_t();
        if(k == -1) break; // timed out
    }
}

// *****************************************************************************
// reads characters up to ACK and returns them, supplu buffer for return
// string, returns with number of charatcers in buffer
// *****************************************************************************
int SV3::read(char *buf)
{
char k,v=0;

    while(k != _ack) {
        k = getch_t();
        if((k == -1) || (k == _ack)) break; // timout or ack
        *(buf++)=k;
        *buf=0; // terminate each time        
        v++;        
    }
    return v;
}

// *****************************************************************************
// returns value as an integer
// *****************************************************************************
int SV3::id()
{
char buf[10];
    flush();
    send("D");
    putch('\r');
    if(read(buf)) return atoi(buf);
    else return 0;
}

// *****************************************************************************
// returns string for firmware in the form h.l, supply buffer to put value in
// *****************************************************************************
void SV3::firmware(char *buf)
{
    flush();
    *buf=0; // just in case 
    send("V");
    putch('\r');
    read(buf);
}

// *****************************************************************************
// resets device
// *****************************************************************************
void SV3::reset()
{
    send("C");
    putch('\r');
    delay(100);
}

// =============================================================================
// EEPROM
// =============================================================================
// =============================================================================
// *****************************************************************************
// reads eeprom, from address to address and places result in a string buffer
// maximum 40 charatcers. The result is presented as a comma separated string
// of decimal numbers
// *****************************************************************************
void SV3::readEE(char from, char bytes, char *result)
{
char b[120];
    if(bytes > 38) return;
    if(bytes+from > 255) return;
    sprintf(b,"R%d,%d\r",from,bytes);
    send(b);
    read(result);
}

// *****************************************************************************
// write a single value to eeprom location is the address of the eeprom that
// will have the range 0 to 255 - no checking is done. 
// *****************************************************************************
void SV3::writeEE(char location, char value)
{
char b[12];
    sprintf(b,"W%d,%d\r",location,value);
    send(b);
    delay(50);
    wack();
}
